package xyz.lychee.lagfixer.modules;

import com.google.common.collect.ImmutableList;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.block.data.BlockData;
import org.bukkit.block.data.type.Leaves;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.HandlerList;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.LeavesDecayEvent;
import xyz.lychee.lagfixer.LagFixer;
import xyz.lychee.lagfixer.managers.ModuleManager;
import xyz.lychee.lagfixer.objects.AbstractModule;
import xyz.lychee.lagfixer.objects.AbstractModule.Impact;

public class InstantLeafDecayModule extends AbstractModule implements Listener {
   private static final EnumSet<BlockFace> FACES;
   private boolean dropItems;
   private boolean leavesDecay;
   private int treeDistance;

   public InstantLeafDecayModule(LagFixer plugin, ModuleManager manager) {
      super(plugin, manager, Impact.LOW, "InstantLeafDecay", ImmutableList.of("Ensures instant leaf removal, reducing leaf blocks for ticking.", "Vital for server performance by eliminating gradual leaf block processing.", "Optimizes server resources for smoother gameplay without decay overhead.", "Ideal for servers with forestry, managing leaf block accumulation."), "eyJ0ZXh0dXJlcyI6eyJTS0lOIjp7InVybCI6Imh0dHA6Ly90ZXh0dXJlcy5taW5lY3JhZnQubmV0L3RleHR1cmUvOWYyZDE0NjkyZDhiMGUzNTI2YTZmYWY0MjY2NzI3YmQwMmFhYTdiMDUyN2IxODVhY2Y3ZjBhYTY2NzkzZmZkYyJ9fX0=");
   }

   @EventHandler(
      priority = EventPriority.HIGHEST,
      ignoreCancelled = true
   )
   public void onBlockBreak(BlockBreakEvent e) {
      Block block = e.getBlock();
      if (this.isValidLeaf(block)) {
         this.breakLeaves(block);
      }

   }

   @EventHandler(
      priority = EventPriority.HIGHEST,
      ignoreCancelled = true
   )
   public void onLeavesDecay(LeavesDecayEvent e) {
      e.setCancelled(true);
      if (this.leavesDecay) {
         Block block = e.getBlock();
         if (this.isValidLeaf(block)) {
            this.breakLeaves(block);
         }
      }

   }

   private void breakLeaves(Block block) {
      Deque<Block> stack = new ArrayDeque();
      stack.push(block);
      Set<Block> visited = new HashSet();

      while(true) {
         Block currentBlock;
         do {
            if (stack.isEmpty()) {
               return;
            }

            currentBlock = (Block)stack.pop();
         } while(!visited.add(currentBlock));

         if (this.dropItems) {
            currentBlock.breakNaturally();
         } else {
            currentBlock.setType(Material.AIR, false);
         }

         Iterator var5 = FACES.iterator();

         while(var5.hasNext()) {
            BlockFace face = (BlockFace)var5.next();
            Block neighborBlock = currentBlock.getRelative(face);
            if (this.isValidLeaf(neighborBlock) && !visited.contains(neighborBlock)) {
               stack.push(neighborBlock);
            }
         }
      }
   }

   private boolean isValidLeaf(Block block) {
      BlockData blockData = block.getBlockData();
      if (!(blockData instanceof Leaves)) {
         return false;
      } else {
         Leaves leaves = (Leaves)blockData;
         return leaves.getDistance() >= this.treeDistance && !leaves.isPersistent();
      }
   }

   public void load() {
      Bukkit.getPluginManager().registerEvents(this, this.getPlugin());
   }

   public boolean loadConfig() {
      this.dropItems = this.getSection().getBoolean("drop_items");
      this.treeDistance = this.getSection().getInt("tree_distance");
      this.leavesDecay = this.getSection().getBoolean("leaves_decay");
      return true;
   }

   public void disable() {
      HandlerList.unregisterAll(this);
   }

   static {
      FACES = EnumSet.complementOf(EnumSet.of(BlockFace.SELF));
   }
}
